(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     13915,        422]
NotebookOptionsPosition[     12558,        376]
NotebookOutlinePosition[     12900,        391]
CellTagsIndexPosition[     12857,        388]
WindowFrame->Normal
ContainsDynamic->False*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["J/Link Activity: Making Music with JFugue", "Title"],

Cell["\<\
From the JFugue User Guide:
\tJFugue is a Java API for Music Programming. With JFugue, you can program \
music quickly and easily, and without knowing MIDI.\
\>", "Text"],

Cell["\<\
JFugue can be obtained from www.jfugue.org. This notebook is bundled with the \
complete JFugue package, including documentation.\
\>", "Text",
 CellChangeTimes->{{3.369649349854771*^9, 3.3696493680809793`*^9}, 
   3.3696494311516705`*^9}],

Cell["\<\
To use JFugue, you create Pattern objects that consist of sequences of notes, \
rests, other patterns, etc. You then play these patterns using a Player \
object.\
\>", "Text"],

Cell["First, setup J/Link:", "Text"],

Cell["\<\
Needs[\"JLink`\"];
InstallJava[];\
\>", "Input",
 CellChangeTimes->{{3.369649391204229*^9, 3.369649424582224*^9}}],

Cell["\<\
To allow J/Link to find the JFugue Java classes, you need to add their \
location to the Java class path. This is only necessary because JFugue is not \
installed into a standard location on the lab computers:\
\>", "Text",
 CellChangeTimes->{{3.3696494406152782`*^9, 3.369649544204232*^9}}],

Cell["AddToClassPath[ToFileName[NotebookDirectory[], \"Java\"]];", "Input",
 CellChangeTimes->{{3.369649391204229*^9, 3.3696494366395617`*^9}}],

Cell["Here is a trivial Java sample program from the JFugue docs:", "Text"],

Cell["\<\
\t\tPlayer player = new Player();
\t\tPattern pattern = new Pattern(\"C D E F G A B\");
\t\tplayer.play(pattern);\
\>", "Program"],

Cell[TextData[{
 "To get started, let's convert this program to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Note that the Player and Pattern classes are in the org.jfugue \
namespace."
}], "Text"],

Cell["\<\
player = JavaNew[\"org.jfugue.Player\"];
pattern = JavaNew[\"org.jfugue.Pattern\", \"C D E F G A B C6\"];
player@play[pattern]\
\>", "Input"],

Cell["\<\
That's all you need to be able to do to program using J/Link.\
\>", "Text"],

Cell["\<\
You will notice the C6 above. The 6 refers to the octave. The default is 5, \
which is the octave starting at middle C. You can use w, h, q, i, etc. to \
specify note durations (whole, half, quarter, eighth). R stands for a rest. \
This scale has some rests and different note durations. Note that we reuse \
the player object created earlier.\
\>", "Text"],

Cell["\<\
pattern = JavaNew[\"org.jfugue.Pattern\", \"C5q D5q E5q F5i. G5i A5q B5q Rq \
C6i\"];
player@play[pattern]\
\>", "Input"],

Cell["\<\
If you are familiar with music notation you will recognize that the \"i.\" \
above stands for a dotted eighth note (one and a half times the length of a \
normal eighth note).\
\>", "Text"],

Cell["\<\
Continue to experiment with JFugue. You can specify chords, different \
instrument voices, drum tracks to go along with your music, multiple voices \
together, etc.\
\>", "Text",
 CellChangeTimes->{{3.3696497671948767`*^9, 3.3696497671948767`*^9}, {
  3.3696498506048145`*^9, 3.369649859006896*^9}}],

Cell[TextData[{
 "Here are links to the JFugue ",
 StyleBox[ButtonBox["User Guide",
  BaseStyle->"Hyperlink",
  Evaluator->Automatic,
  ButtonData:>{
    URL[
     ToFileName[{
       NotebookDirectory[], "jfugue-docs"}, "JFugue-UserGuide.html"]], None}],
   "MR"],
 " and ",
 StyleBox[ButtonBox["JavaDocs",
  BaseStyle->"Hyperlink",
  Evaluator->Automatic,
  ButtonData:>{
    URL[
     ToFileName[{
       NotebookDirectory[], "jfugue-docs", "javadoc"}, "index.html"]], None}],
   "MR"],
 " (a class-by-class listing of methods)."
}], "Text"],

Cell["\<\
Here is a more complicated example: \"Frere Jacques\" played as a 3-part \
round in 3 instruments. Note how the final song, itself just a Pattern \
object, is built out of nested Pattern objects representing the voices, and \
that each of those is built with the same subpatterns.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"frereJacquesMelody", " ", "=", " ", 
    RowBox[{"JavaNew", "[", 
     RowBox[{
     "\"\<org.jfugue.Pattern\>\"", ",", " ", 
      "\"\<C5q D5q E5q C5q C5q D5q E5q C5q E5q F5q G5h E5q F5q G5h G5i A5i \
G5i F5i E5q C5q G5i A5i G5i F5i E5q C5q C5q G4q C5h C5q G4q C5h\>\""}], 
     "]"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"doubleMeasureRest", "=", 
    RowBox[{"JavaNew", "[", 
     RowBox[{"\"\<org.jfugue.Pattern\>\"", ",", "\"\<Rw Rw\>\""}], "]"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"round1", " ", "=", " ", 
    RowBox[{"JavaNew", "[", 
     RowBox[{"\"\<org.jfugue.Pattern\>\"", ",", "\"\<V0 I[FLUTE]\>\""}], 
     "]"}]}], ";"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{
    RowBox[{"round1", "@", 
     RowBox[{"add", "[", "frereJacquesMelody", "]"}]}], ";"}],
   FormatType->StandardForm], 
  StyleBox[" ",
   FormatType->StandardForm], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 

 StyleBox[
  RowBox[{
   RowBox[{"round2", " ", "=", " ", 
    RowBox[{"JavaNew", "[", 
     RowBox[{"\"\<org.jfugue.Pattern\>\"", ",", "\"\<V1 I[ALTO_SAX]\>\""}], 
     "]"}]}], ";"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"round2", "@", 
    RowBox[{"add", "[", "doubleMeasureRest", "]"}]}], ";"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{
    RowBox[{"round2", "@", 
     RowBox[{"add", "[", "frereJacquesMelody", "]"}]}], ";"}], 
   "\[IndentingNewLine]"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"round3", " ", "=", " ", 
    RowBox[{"JavaNew", "[", 
     RowBox[{"\"\<org.jfugue.Pattern\>\"", ",", "\"\<V2 I[CLARINET]\>\""}], 
     "]"}]}], ";"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"round3", "@", 
    RowBox[{"add", "[", "doubleMeasureRest", "]"}]}], ";"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"round3", "@", 
    RowBox[{"add", "[", "doubleMeasureRest", "]"}]}], ";"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{
    RowBox[{"round3", "@", 
     RowBox[{"add", "[", "frereJacquesMelody", "]"}]}], ";"}],
   FormatType->StandardForm], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 

 StyleBox[
  RowBox[{
   RowBox[{"fullSong", " ", "=", " ", 
    RowBox[{"JavaNew", "[", "\"\<org.jfugue.Pattern\>\"", "]"}]}], ";"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"fullSong", "@", 
    RowBox[{"add", "[", "round1", "]"}]}], ";"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 StyleBox[
  RowBox[{
   RowBox[{"fullSong", "@", 
    RowBox[{"add", "[", "round2", "]"}]}], ";"}],
  FormatType->StandardForm], "\[IndentingNewLine]", 
 RowBox[{
  StyleBox[
   RowBox[{
    RowBox[{"fullSong", "@", 
     RowBox[{"add", "[", "round3", "]"}]}], ";"}],
   FormatType->StandardForm], 
  StyleBox[" ",
   FormatType->StandardForm], "\n"}], "\[IndentingNewLine]", 
 RowBox[{"player", "@", 
  RowBox[{"play", "[", "fullSong", "]"}]}]}], "Input"],

Cell[CellGroupData[{

Cell["Some ideas:", "Subsection"],

Cell["Write a program that generates random songs", "Text"],

Cell["Write a program that plays any song as an n-part round", "Text"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Write a program that generates music that is in some \
way the product of a cellular automaton or other algorithm.\n\n(An example is \
in this closed section)",
 FontWeight->"Plain"]], "Subsubsection"],

Cell["\<\
The cellularSong function plays a song where each beat represents a line in \
the ouput of a cellular automaton. Think of each row of a cellular automaton \
output as being the white keys of a keyboard. A 1 at a given position means \
that key is pressed. We position middle C on the center of the row, and go at \
most one octave in each direction [Actually, this program has a bug and does \
not properly center on middle C for all types of input. Fix it if you like].\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"cellularSong", "[", 
   RowBox[{"rule_Integer", ",", "steps_Integer", ",", 
    RowBox[{"startingPattern_List", ":", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "1", "}"}], ",", "0"}], "}"}]}]}], "]"}], ":=", 
  "\[IndentingNewLine]", 
  RowBox[{"JavaBlock", "[", "\[IndentingNewLine]", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "ca", ",", "range", ",", "midpoint", ",", "twoOctaveRange", ",", 
       "noteRules", ",", "notes", ",", "chords", ",", "caSong"}], "}"}], ",", 
     
     RowBox[{
      RowBox[{"ca", "=", 
       RowBox[{"CellularAutomaton", "[", 
        RowBox[{"rule", ",", "startingPattern", ",", "steps"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"range", "=", 
       RowBox[{"Length", "[", 
        RowBox[{"First", "[", "ca", "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"midpoint", "=", 
       RowBox[{"Round", "[", 
        FractionBox[
         RowBox[{"range", "+", "1"}], "2"], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"twoOctaveRange", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Take", "[", 
          RowBox[{"#1", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"midpoint", "-", 
              RowBox[{"Min", "[", 
               RowBox[{"7", ",", 
                RowBox[{"Round", "[", 
                 FractionBox[
                  RowBox[{"range", "-", "0.1"}], "2"], "]"}]}], "]"}]}], ",", 
             
             RowBox[{"midpoint", "+", 
              RowBox[{"Min", "[", 
               RowBox[{"7", ",", 
                RowBox[{"Round", "[", 
                 FractionBox[
                  RowBox[{"range", "-", " ", "0.1"}], "2"], "]"}]}], 
               "]"}]}]}], "}"}]}], "]"}], "&"}], "/@", "ca"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"noteRules", "=", 
       RowBox[{"Thread", "[", 
        RowBox[{
         RowBox[{"Range", "[", "15", "]"}], "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
          "\"\<C4\>\"", ",", "\"\<D4\>\"", ",", "\"\<E4\>\"", ",", 
           "\"\<F4\>\"", ",", "\"\<G4\>\"", ",", "\"\<A4\>\"", ",", 
           "\"\<B4\>\"", ",", "\"\<C5\>\"", ",", "\"\<D5\>\"", ",", 
           "\"\<E5\>\"", ",", "\"\<F5\>\"", ",", "\"\<G5\>\"", ",", 
           "\"\<A5\>\"", ",", "\"\<B5\>\"", ",", "\"\<C6\>\""}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"notes", "=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Flatten", "[", 
           RowBox[{"Position", "[", 
            RowBox[{"#1", ",", "1"}], "]"}], "]"}], "&"}], "/@", 
         "twoOctaveRange"}], "/.", "\[InvisibleSpace]", "noteRules"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"chords", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Fold", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"#1", "<>", "\"\<+\>\"", "<>", "#2"}], "&"}], ",", 
           RowBox[{"First", "[", "#1", "]"}], ",", 
           RowBox[{"Rest", "[", "#1", "]"}]}], "]"}], "&"}], "/@", 
        "notes"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"caSong", "=", 
       RowBox[{"JavaNew", "[", "\"\<org.jfugue.Pattern\>\"", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"caSong", "@", 
         RowBox[{"add", "[", 
          RowBox[{"JavaNew", "[", 
           RowBox[{"\"\<org.jfugue.Pattern\>\"", ",", "#1"}], "]"}], "]"}]}], 
        "&"}], "/@", "chords"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"JavaNew", "[", "\"\<org.jfugue.Player\>\"", "]"}], "@", 
       RowBox[{"play", "[", "caSong", "]"}]}]}]}], "\[IndentingNewLine]", 
    "]"}], "\[IndentingNewLine]", "]"}]}]], "Input"],

Cell["20 steps of the rule 30 CA:", "Text"],

Cell["cellularSong[30, 20]", "Input"]
}, Closed]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{970, 630},
WindowMargins->{{7, Automatic}, {Automatic, 3}},
FrontEndVersion->"6.0 for Microsoft Windows (32-bit) (October 10, 2006)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[590, 23, 58, 0, 83, "Title"],
Cell[651, 25, 180, 4, 47, "Text"],
Cell[834, 31, 249, 5, 29, "Text"],
Cell[1086, 38, 185, 4, 29, "Text"],
Cell[1274, 44, 36, 0, 29, "Text"],
Cell[1313, 46, 124, 4, 45, "Input"],
Cell[1440, 52, 301, 5, 47, "Text"],
Cell[1744, 59, 143, 1, 28, "Input"],
Cell[1890, 62, 75, 0, 29, "Text"],
Cell[1968, 64, 140, 4, 72, "Program"],
Cell[2111, 70, 208, 6, 29, "Text"],
Cell[2322, 78, 151, 4, 62, "Input"],
Cell[2476, 84, 85, 2, 29, "Text"],
Cell[2564, 88, 367, 6, 47, "Text"],
Cell[2934, 96, 131, 4, 45, "Input"],
Cell[3068, 102, 199, 4, 29, "Text"],
Cell[3270, 108, 309, 6, 29, "Text"],
Cell[3582, 116, 544, 20, 29, "Text"],
Cell[4129, 138, 304, 5, 47, "Text"],
Cell[4436, 145, 3284, 99, 488, "Input"],
Cell[CellGroupData[{
Cell[7745, 248, 33, 0, 36, "Subsection"],
Cell[7781, 250, 59, 0, 29, "Text"],
Cell[7843, 252, 70, 0, 29, "Text"],
Cell[CellGroupData[{
Cell[7938, 256, 225, 3, 59, "Subsubsection"],
Cell[8166, 261, 494, 7, 71, "Text"],
Cell[8663, 270, 3769, 97, 362, "Input"],
Cell[12435, 369, 43, 0, 33, "Text"],
Cell[12481, 371, 37, 0, 30, "Input"]
}, Closed]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
